#!/usr/bin/python
# vim:expandtab:textwidth=0:autoindent:tabstop=4:shiftwidth=4:filetype=python:

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""module

some docs here eventually.
"""

from __future__ import generators

# import arranged alphabetically
import commands
import os
import stat
import sys

# local modules
import firmwaretools.package as package
import firmware_addon_dell.biosHdr as biosHdr
import firmware_addon_dell.svm as svm
from firmwaretools.trace_decorator import trace, dprint, decorateAllFunctions
import lsi_raid_inventory

class PercSuper(package.RepositoryPackage):
    def __init__(self, *args, **kargs):
        super(PercSuper, self).__init__(*args, **kargs)
        #self.capabilities['can_downgrade'] = True
        #self.capabilities['can_reflash'] = True

    def genericInstall(self, exe, adapName, downgradeOption="-r", upgradeOption=""):
        self.status = "in_progress"
        installDevice = self.getCurrentInstallDevice()
        option = upgradeOption
        if self.compareVersion(installDevice) <= 0:
            #print "DOWNGRADE DETECTED"
            option = downgradeOption

        #print "Debug: package.version: %s" % self.version
        #print "Debug: device.version: %s" % installDevice.version
        #print "Debug: compare: %s" % self.compareVersion(installDevice)
        #print "Debug: %s  *** %s *** %s ***" % (downgradeOption, upgradeOption, option)
        #print "INSTALLING %s FIRMWARE: %s - %s" % (adapName, self.name, self.version)

        instStr = "cd %s;LD_LIBRARY_PATH=/usr/libexec/dell-lsiflash /usr/libexec/dell-lsiflash/%s %s " % (self.path, exe, option)
        #print instStr
        status, output = commands.getstatusoutput(instStr)
        dprint(output)
        if status:
            self.status = "failed"
            raise package.InstallError(output)

        self.status = "success"
        return 1

class Perc4Package(PercSuper):
    def install(self):
        return self.genericInstall("linflash.bin -s -r", "LSI PERC4", downgradeOption="-f", upgradeOption="-v")

class Perc5iPackage(PercSuper):
    def install(self):
        return self.genericInstall("perc5int -a", "LSI PERC5i")

class Perc5ePackage(PercSuper):
    def install(self):
        return self.genericInstall("perc5 -a", "LSI PERC5e")

class Perc5eAdapPackage(PercSuper):
    def install(self):
        return self.genericInstall("perc5iadap -a", "LSI PERC5e")

class Sas5irPackage(PercSuper):
    def install(self):
        return self.genericInstall("sas5iradapter -a", "LSI SAS 5ir")

class Sas5irintPackage(PercSuper):
    def install(self):
        return self.genericInstall("sas5irint -a", "LSI SAS 5ir INT")


for i in (Perc4Package, Perc5iPackage, Perc5ePackage, Perc5eAdapPackage, Sas5irPackage, Sas5irintPackage):
    i.install = trace(i.install)

# standard entry point -- Bootstrap
#  NOT NEEDED: no bootstrap here, as firmware-tools lspci module provides bootstrap for this module.
#def BootstrapGenerator():  pass

# standard entry point -- Inventory
#
def InventoryGenerator():
    ret = createMegadevFile()
    if ret:
        for p in InventoryPerc4():
            yield p

    try:
        for p in InventoryPerc5AndSas5():
            yield p
    except:
        import traceback
        traceback.print_exc()


# not optimal. Need to internalize this at some point in future.
def InventoryPerc5AndSas5():
    cmds = ('perc5int', 'perc5', 'perc5iadap', 'sas5irint', 'sas5iradapter')
    cmdstr = "LD_LIBRARY_PATH=/usr/libexec/dell-lsiflash/ /usr/libexec/dell-lsiflash/%s -i"
    for cmd in cmds:
        (status, inventoryXml) = commands.getstatusoutput(cmdstr % cmd)
        for pkg in svm.genPackagesFromSvmXml(inventoryXml):
            yield pkg

def InventoryPerc4():
    n = lsi_raid_inventory.getNumberAdapters()
    for i in range(n):
        inv = lsi_raid_inventory.getAdapterDetails(i)
        p = package.PciDevice(
            displayname = inv["product"],
            name = "pci_firmware(ven_0x%04x_dev_0x%04x_subven_0x%04x_subdev_0x%04x)".lower() %
                (inv["pci_id"][0], inv["pci_id"][1], inv["pci_id"][2], inv["pci_id"][3]),
            version = inv["fw_ver"].lower(),
            compareStrategy = biosHdr.compareVersions,
            pciDbdf = inv["pci_dbdf"]
            )
        yield p

def createMegadevFile():
    # create megadev0
    retval = 0
    fd = open("/proc/devices", "r")
    major = 0
    while 1:
        line = fd.readline()
        if line == "": break
        if " megadev\n" in line:
            major = int(line.split()[0])
    fd.close()
    if major:
        try:
            os.unlink("/dev/megadev0")
        except OSError:
            pass
        os.mknod( "/dev/megadev0", (0600 | stat.S_IFCHR), os.makedev(major, 0))
        retval = 1
    return retval


decorateAllFunctions(sys.modules[__name__])

